@@ -1,10 +1,10 @@
 /*
     MIDI Sequencer C++ library
-    Copyright (C) 2006-2015, Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2006-2021, Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This library is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
@@ -12,26 +12,35 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+    You should have received a copy of the GNU General Public License
+    along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef DRUMSTICK_ALSAEVENT_H
 #define DRUMSTICK_ALSAEVENT_H
 
-#include "drumstickcommon.h"
+extern "C" {
+    #include <alsa/asoundlib.h>
+}
+
+#include <QObject>
 #include <QEvent>
+#include "macros.h"
+
+namespace drumstick { namespace ALSA {
 
 /**
  * @file alsaevent.h
  * Classes managing ALSA Sequencer events.
  *
- * @defgroup ALSAEvent ALSA Sequencer Events
+ * @addtogroup ALSAEvent ALSA Sequencer Events
  * @{
  */
 
-namespace drumstick {
+/**
+ * 8-bit unsigned number to be used as a MIDI message parameter
+ */
+typedef quint8 MidiByte;
 
 /**
  * Constant SequencerEventType is the QEvent::type() of any SequencerEvent
@@ -39,11 +48,6 @@ namespace drumstick {
  */
 const QEvent::Type SequencerEventType = QEvent::Type(QEvent::User + 4154); // :-)
 
-/**
- * Macro to declare a virtual clone() method for SequencerEvent and derived classes.
- */
-#define CLONE_EVENT_DECLARATION(T) virtual T* clone() { return new T(&m_event); }
-
 /**
  * Base class for the event's hierarchy
  *
@@ -55,9 +59,7 @@ class DRUMSTICK_EXPORT SequencerEvent : public QEvent
 public:
     SequencerEvent();
     SequencerEvent(const SequencerEvent& other);
-    SequencerEvent(snd_seq_event_t* event);
-    /** Destructor */
-    virtual ~SequencerEvent() {}
+    explicit SequencerEvent(const snd_seq_event_t* event);
 
     SequencerEvent& operator=(const SequencerEvent& other);
     void setSequencerType(const snd_seq_event_type_t eventType);
@@ -128,16 +130,14 @@ public:
     static bool isClient(const SequencerEvent* event);
     static bool isConnectionChange(const SequencerEvent* event);
     static bool isChannel(const SequencerEvent* event);
-
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(SequencerEvent);
+    virtual SequencerEvent* clone() const;
 
 protected:
     void free() __attribute__((deprecated));
 
     /**
      * ALSA sequencer event record.
-     * @see http://www.alsa-project.org/alsa-doc/alsa-lib/structsnd__seq__event.html
+     * @see https://www.alsa-project.org/alsa-doc/alsa-lib/structsnd__seq__event.html
      */
     snd_seq_event_t m_event;
 };
@@ -150,8 +150,11 @@ class DRUMSTICK_EXPORT ChannelEvent : public SequencerEvent
 public:
     /** Default constructor */
     ChannelEvent() : SequencerEvent() {}
-    /** Constructor from an ALSA event record */
-    ChannelEvent(snd_seq_event_t* event) : SequencerEvent(event) {}
+    /**
+     * Constructor from an ALSA event record
+     * @param event an ALSA event record
+     */
+    explicit ChannelEvent(const snd_seq_event_t* event) : SequencerEvent(event) {}
     /**
      * Sets the channel of the event
      * @param c A channel, between 0 and 15.
@@ -164,6 +167,8 @@ public:
      * @see setChannel()
      */
     int getChannel() const { return m_event.data.note.channel; }
+
+    virtual ChannelEvent* clone() const override;
 };
 
 /**
@@ -174,8 +179,11 @@ class DRUMSTICK_EXPORT KeyEvent : public ChannelEvent
 public:
     /** Default constructor */
     KeyEvent() : ChannelEvent() {}
-    /** Constructor from an ALSA event record */
-    KeyEvent(snd_seq_event_t* event) : ChannelEvent(event) {}
+    /**
+     * Constructor from an ALSA event record
+     * @param event an ALSA event record
+     */
+    explicit KeyEvent(const snd_seq_event_t* event) : ChannelEvent(event) {}
     /**
      * Gets the MIDI note of this event.
      * @return The event's MIDI note.
@@ -200,6 +208,8 @@ public:
      * @see getVelocity()
      */
     void setVelocity(const MidiByte b) { m_event.data.note.velocity = b; }
+
+    virtual KeyEvent* clone() const override;
 };
 
 /**
@@ -213,8 +223,18 @@ class DRUMSTICK_EXPORT NoteEvent : public KeyEvent
 public:
     /** Default constructor */
     NoteEvent() : KeyEvent() { m_event.type = SND_SEQ_EVENT_NOTE; }
-    /** Constructor from an ALSA event record */
-    NoteEvent(snd_seq_event_t* event) : KeyEvent(event) {}
+    /**
+     * Constructor from an ALSA event record
+     * @param event an ALSA event record
+     */
+    explicit NoteEvent(const snd_seq_event_t* event) : KeyEvent(event) {}
+    /**
+     * Constructor
+     * @param ch MIDI Channel
+     * @param key a MIDI note number
+     * @param vel a MIDI velocity value
+     * @param dur duration of the note in ticks
+     */
     NoteEvent(const int ch, const int key, const int vel, const int dur);
     /**
      * Gets the note's duration
@@ -228,8 +248,8 @@ public:
      * @see getDuration()
      */
     void setDuration(const ulong d) { m_event.data.note.duration = d; }
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(NoteEvent)
+
+    virtual NoteEvent* clone() const override;
 };
 
 /**
@@ -240,11 +260,19 @@ class DRUMSTICK_EXPORT NoteOnEvent : public KeyEvent
 public:
     /** Default constructor */
     NoteOnEvent() : KeyEvent() { m_event.type = SND_SEQ_EVENT_NOTEON; }
-    /** Constructor from an ALSA event record */
-    NoteOnEvent(snd_seq_event_t* event) : KeyEvent(event) {}
+    /**
+     * Constructor from an ALSA event record
+     * @param event an ALSA event record
+     */
+    explicit NoteOnEvent(const snd_seq_event_t* event) : KeyEvent(event) {}
+    /**
+     * Constructor
+     * @param ch MIDI Channel
+     * @param key a MIDI note number
+     * @param vel a MIDI velocity value
+     */
     NoteOnEvent(const int ch, const int key, const int vel);
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(NoteOnEvent)
+    virtual NoteOnEvent* clone() const override;
 };
 
 /**
@@ -255,11 +283,19 @@ class DRUMSTICK_EXPORT NoteOffEvent : public KeyEvent
 public:
     /** Default constructor */
     NoteOffEvent() : KeyEvent() { m_event.type = SND_SEQ_EVENT_NOTEOFF; }
-    /** Constructor from an ALSA event record */
-    NoteOffEvent(snd_seq_event_t* event) : KeyEvent(event) {}
+    /**
+     * Constructor from an ALSA event record
+     * @param event an ALSA event record
+     */
+    explicit NoteOffEvent(const snd_seq_event_t* event) : KeyEvent(event) {}
+    /**
+     * Constructor
+     * @param ch MIDI Channel
+     * @param key a MIDI note number
+     * @param vel a MIDI velocity value
+     */
     NoteOffEvent(const int ch, const int key, const int vel);
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(NoteOffEvent)
+    virtual NoteOffEvent* clone() const override;
 };
 
 /**
@@ -270,11 +306,19 @@ class DRUMSTICK_EXPORT KeyPressEvent : public KeyEvent
 public:
     /** Default constructor */
     KeyPressEvent() : KeyEvent() { m_event.type = SND_SEQ_EVENT_KEYPRESS; }
-    /** Constructor from an ALSA event record */
-    KeyPressEvent(snd_seq_event_t* event) : KeyEvent(event) {}
+    /**
+     * Constructor from an ALSA event record
+     * @param event an ALSA event record
+     */
+    explicit KeyPressEvent(const snd_seq_event_t* event) : KeyEvent(event) {}
+    /**
+     * Constructor
+     * @param ch MIDI Channel
+     * @param key a MIDI note number
+     * @param vel a MIDI velocity value
+     */
     KeyPressEvent(const int ch, const int key, const int vel);
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(KeyPressEvent)
+    virtual KeyPressEvent* clone() const override;
 };
 
 /**
@@ -285,8 +329,17 @@ class DRUMSTICK_EXPORT ControllerEvent : public ChannelEvent
 public:
     /** Default constructor */
     ControllerEvent() : ChannelEvent() {}
-    /** Constructor from an ALSA event record */
-    ControllerEvent(snd_seq_event_t* event) : ChannelEvent(event) {}
+    /**
+     * Constructor from an ALSA event record
+     * @param event an ALSA event record
+     */
+    explicit ControllerEvent(const snd_seq_event_t* event) : ChannelEvent(event) {}
+    /**
+     * Constructor
+     * @param ch MIDI Channel
+     * @param cc a MIDI controller
+     * @param val a value
+     */
     ControllerEvent(const int ch, const int cc, const int val);
     /**
      * Gets the controller event's parameter.
@@ -312,8 +365,7 @@ public:
      * @see getValue()
      */
     void setValue( const int v ) { m_event.data.control.value = v; }
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(ControllerEvent)
+    virtual ControllerEvent* clone() const override;
 };
 
 /**
@@ -324,15 +376,28 @@ class DRUMSTICK_EXPORT ProgramChangeEvent : public ChannelEvent
 public:
     /** Default constructor */
     ProgramChangeEvent() : ChannelEvent() { m_event.type = SND_SEQ_EVENT_PGMCHANGE; }
-    /** Constructor from an ALSA event record */
-    ProgramChangeEvent(snd_seq_event_t* event) : ChannelEvent(event) {}
+    /**
+     * Constructor from an ALSA event record
+     * @param event an ALSA event record
+     */
+    explicit ProgramChangeEvent(const snd_seq_event_t* event) : ChannelEvent(event) {}
+    /**
+     * Constructor
+     * @param ch MIDI Channel
+     * @param val a value
+     */
     ProgramChangeEvent(const int ch, const int val);
-    /** Gets the MIDI program number */
+    /**
+     * Gets the MIDI program number
+     * @return the MIDI program number
+     */
     int getValue() const { return m_event.data.control.value; }
-    /** Sets the MIDI program number */
+    /**
+     * Sets the MIDI program number
+     * @param v the MIDI program number
+     */
     void setValue( const int v ) { m_event.data.control.value = v; }
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(ProgramChangeEvent)
+    virtual ProgramChangeEvent* clone() const override;
 };
 
 /**
@@ -343,15 +408,28 @@ class DRUMSTICK_EXPORT PitchBendEvent : public ChannelEvent
 public:
     /** Default constructor */
     PitchBendEvent() : ChannelEvent() { m_event.type = SND_SEQ_EVENT_PITCHBEND; }
-    /** Constructor from an ALSA event record */
-    PitchBendEvent(snd_seq_event_t* event) : ChannelEvent(event) {}
+    /**
+     * Constructor from an ALSA event record
+     * @param event an ALSA event record
+     */
+    explicit PitchBendEvent(const snd_seq_event_t* event) : ChannelEvent(event) {}
+    /**
+     * Constructor
+     * @param ch MIDI Channel
+     * @param val a value
+     */
     PitchBendEvent(const int ch, const int val);
-    /** Gets the MIDI pitch bend value, zero centered from -8192 to 8191 */
+    /**
+     * Gets the MIDI pitch bend value, zero centered from -8192 to 8191
+     * @return the MIDI pitch bend value
+     */
     int getValue() const { return m_event.data.control.value; }
-    /** Sets the MIDI pitch bend value, zero centered from -8192 to 8191  */
+    /**
+     * Sets the MIDI pitch bend value, zero centered from -8192 to 8191
+     * @param v the MIDI pitch bend value
+     */
     void setValue( const int v ) { m_event.data.control.value = v; }
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(PitchBendEvent)
+    virtual PitchBendEvent* clone() const override;
 };
 
 /**
@@ -362,15 +440,28 @@ class DRUMSTICK_EXPORT ChanPressEvent : public ChannelEvent
 public:
     /** Default constructor */
     ChanPressEvent() : ChannelEvent() { m_event.type = SND_SEQ_EVENT_CHANPRESS; }
-    /** Constructor from an ALSA event record */
-    ChanPressEvent(snd_seq_event_t* event) : ChannelEvent(event) {}
-    ChanPressEvent( const int ch, const int val);
-    /** Gets the channel aftertouch value */
+    /**
+     * Constructor from an ALSA event record
+     * @param event an ALSA event record
+     */
+    explicit ChanPressEvent( const snd_seq_event_t* event ) : ChannelEvent(event) {}
+    /**
+     * Constructor
+     * @param ch MIDI Channel
+     * @param val a value
+     */
+    ChanPressEvent( const int ch, const int val );
+    /**
+     * Gets the channel aftertouch value
+     * @return the channel aftertouch value
+     */
     int getValue() const { return m_event.data.control.value; }
-    /** Sets the channel aftertouch value */
+    /**
+     * Sets the channel aftertouch value
+     * @param v the channel aftertouch value
+     */
     void setValue( const int v ) { m_event.data.control.value = v; }
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(ChanPressEvent)
+    virtual ChanPressEvent* clone() const override;
 };
 
 /**
@@ -380,17 +471,22 @@ class DRUMSTICK_EXPORT VariableEvent : public SequencerEvent
 {
 public:
     VariableEvent();
-    VariableEvent(snd_seq_event_t* event);
-    VariableEvent(const QByteArray& data);
+    explicit VariableEvent(const snd_seq_event_t* event);
+    explicit VariableEvent(const QByteArray& data);
     VariableEvent(const VariableEvent& other);
     VariableEvent(const unsigned int datalen, char* dataptr);
     VariableEvent& operator=(const VariableEvent& other);
-    /** Gets the data length */
+    /**
+     * Gets the data length
+     * @return the data length
+     */
     unsigned int getLength() const { return m_event.data.ext.len; }
-    /** Gets the data pointer */
+    /**
+     * Gets the data pointer
+     * @return the data pointer
+     */
     const char* getData() const { return static_cast<const char*>(m_event.data.ext.ptr); }
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(VariableEvent)
+    virtual VariableEvent* clone() const override;
 protected:
     QByteArray m_data;
 };
@@ -402,12 +498,11 @@ class DRUMSTICK_EXPORT SysExEvent : public VariableEvent
 {
 public:
     SysExEvent();
-    SysExEvent(snd_seq_event_t* event);
-    SysExEvent(const QByteArray& data);
+    explicit SysExEvent(const snd_seq_event_t* event);
+    explicit SysExEvent(const QByteArray& data);
     SysExEvent(const SysExEvent& other);
     SysExEvent(const unsigned int datalen, char* dataptr);
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(SysExEvent)
+    virtual SysExEvent* clone() const override;
 };
 
 /**
@@ -420,14 +515,13 @@ class DRUMSTICK_EXPORT TextEvent : public VariableEvent
 {
 public:
     TextEvent();
-    TextEvent(snd_seq_event_t* event);
+    explicit TextEvent(const snd_seq_event_t* event);
     explicit TextEvent(const QString& text, const int textType = 1);
     TextEvent(const TextEvent& other);
     TextEvent(const unsigned int datalen, char* dataptr);
     QString getText() const;
     int getTextType() const;
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(TextEvent)
+    virtual TextEvent* clone() const override;
 protected:
     int m_textType;
 };
@@ -440,11 +534,13 @@ class DRUMSTICK_EXPORT SystemEvent : public SequencerEvent
 public:
     /** Default constructor */
     SystemEvent() : SequencerEvent() {}
-    /** Constructor from an ALSA event record */
-    SystemEvent(snd_seq_event_t* event) : SequencerEvent(event) {}
-    SystemEvent(const snd_seq_event_type_t type);
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(SystemEvent)
+    /**
+     * Constructor from an ALSA event record
+     * @param event an ALSA event record
+     */
+    explicit SystemEvent(const snd_seq_event_t* event) : SequencerEvent(event) {}
+    explicit SystemEvent(const snd_seq_event_type_t type);
+    virtual SystemEvent* clone() const override;
 };
 
 /**
@@ -457,35 +553,73 @@ class DRUMSTICK_EXPORT QueueControlEvent : public SequencerEvent
 public:
     /** Default constructor */
     QueueControlEvent() : SequencerEvent() {}
-    /** Constructor from an ALSA event record */
-    QueueControlEvent(snd_seq_event_t* event) : SequencerEvent(event) {}
+    /**
+     * Constructor from an ALSA event record
+     * @param event an ALSA event record
+     */
+    explicit QueueControlEvent(const snd_seq_event_t* event) : SequencerEvent(event) {}
     QueueControlEvent(const snd_seq_event_type_t type, const int queue, const int value);
-    /** Gets the queue number */
+    /**
+     * Gets the queue number
+     * @return the queue number
+     */
     int getQueue() const { return m_event.data.queue.queue; }
-    /** Sets the queue number */
+    /**
+     * Sets the queue number
+     * @param q the queue number
+     */
     void setQueue(const uchar q) { m_event.data.queue.queue = q; }
-    /** Gets the event's value */
+    /**
+     * Gets the event's value
+     * @return the event's value
+     */
     int getValue() const { return m_event.data.queue.param.value; }
-    /** Sets the event's value */
+    /**
+     * Sets the event's value
+     * @param val the event's value
+     */
     void setValue(const int val) { m_event.data.queue.param.value = val; }
-    /** Gets the queue position */
+    /**
+     * Gets the queue position
+     * @return the queue position
+     */
     uint getPosition() const { return m_event.data.queue.param.position; }
-    /** Sets the queue position */
+    /**
+     * Sets the queue position
+     * @param pos the queue position
+     */
     void setPosition(const uint pos) { m_event.data.queue.param.position = pos; }
-    /** Gets the musical time in ticks */
+    /**
+     * Gets the musical time in ticks
+     * @return the musical time in ticks
+     */
     snd_seq_tick_time_t getTickTime() const { return m_event.data.queue.param.time.tick; }
-    /** Sets the musical time in ticks */
+    /**
+     * Sets the musical time in ticks
+     * @param t the musical time in ticks
+     */
     void setTickTime(const snd_seq_tick_time_t t) { m_event.data.queue.param.time.tick = t; }
-    /** Gets the skew base */
+    /**
+     * Gets the skew base
+     * @return the skew base
+     */
     uint getSkewBase() const { return m_event.data.queue.param.skew.base;  }
-    /** Sets the skew base, should be 65536 */
+    /**
+     * Sets the skew base, should be 65536
+     * @param base the skew base, should be 65536
+     */
     void setSkewBase(const uint base) { m_event.data.queue.param.skew.base = base; }
-    /** Gets the skew value */
+    /**
+     * Gets the skew value
+     * @return the skew value
+     */
     uint getSkewValue() const { return m_event.data.queue.param.skew.value;  }
-    /** Sets the skew value */
+    /**
+     * Sets the skew value
+     * @param val the skew value
+     */
     void setSkewValue(const uint val) {m_event.data.queue.param.skew.value = val; }
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(QueueControlEvent)
+    virtual QueueControlEvent* clone() const override;
 };
 
 /**
@@ -496,15 +630,23 @@ class DRUMSTICK_EXPORT ValueEvent : public SequencerEvent
 public:
     /** Default constructor */
     ValueEvent() : SequencerEvent() {}
-    /** Constructor from an ALSA event record */
-    ValueEvent(snd_seq_event_t* event) : SequencerEvent(event) {}
+    /**
+     * Constructor from an ALSA event record
+     * @param event an ALSA event record
+     */
+    explicit ValueEvent(const snd_seq_event_t* event) : SequencerEvent(event) {}
     ValueEvent(const snd_seq_event_type_t type, const int val);
-    /** Gets the event's value */
+    /**
+     * Gets the event's value
+     * @return the event's value
+     */
     int getValue() const { return m_event.data.control.value; }
-    /** Sets the event's value */
+    /**
+     * Sets the event's value
+     * @param v the event's value
+     */
     void setValue( const int v ) { m_event.data.control.value = v; }
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(ValueEvent)
+    virtual ValueEvent* clone() const override;
 };
 
 /**
@@ -515,11 +657,13 @@ class DRUMSTICK_EXPORT TempoEvent : public QueueControlEvent
 public:
     /** Default constructor */
     TempoEvent() : QueueControlEvent() {}
-    /** Constructor from an ALSA event record */
-    TempoEvent(snd_seq_event_t* event) : QueueControlEvent(event) {}
+    /**
+     * Constructor from an ALSA event record
+     * @param event an ALSA event record
+     */
+    explicit TempoEvent(const snd_seq_event_t* event) : QueueControlEvent(event) {}
     TempoEvent(const int queue, const int tempo);
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(TempoEvent)
+    virtual TempoEvent* clone() const override;
 };
 
 /**
@@ -530,22 +674,42 @@ class DRUMSTICK_EXPORT SubscriptionEvent : public SequencerEvent
 public:
     /** Default constructor */
     SubscriptionEvent() : SequencerEvent() {}
-    /** Constructor from an ALSA event record */
-    SubscriptionEvent(snd_seq_event_t* event) : SequencerEvent(event) {}
-    /** Returns true if the event was a subscribed port */
+    /**
+     * Constructor from an ALSA event record
+     * @param event an ALSA event record
+     */
+    explicit SubscriptionEvent(const snd_seq_event_t* event) : SequencerEvent(event) {}
+    /**
+     * Returns true if the event was a subscribed port
+     * @return whether the event was a subscribed port
+     */
     bool subscribed() const { return (m_event.type == SND_SEQ_EVENT_PORT_SUBSCRIBED); }
-    /** Returns true if the event was an unsubscribed port */
+    /**
+     * Returns true if the event was an unsubscribed port
+     * @return whether the event was an unsubscribed port
+     */
     bool unsubscribed() const { return (m_event.type == SND_SEQ_EVENT_PORT_UNSUBSCRIBED); }
-    /** Gets the sender client number */
+    /**
+     * Gets the sender client number
+     * @return the sender client number
+     */
     int getSenderClient() const { return m_event.data.connect.sender.client; }
-    /** Gets the sender port number */
+    /**
+     * Gets the sender port number
+     * @return the sender port number
+     */
     int getSenderPort() const { return m_event.data.connect.sender.port; }
-    /** Gets the destination client number */
+    /**
+     * Gets the destination client number
+     * @return the destination client number
+     */
     int getDestClient() const { return m_event.data.connect.dest.client; }
-    /** Gets the destination port number */
+    /**
+     * Gets the destination port number
+     * @return the destination port number
+     */
     int getDestPort() const { return m_event.data.connect.dest.port; }
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(SubscriptionEvent)
+    virtual SubscriptionEvent* clone() const override;
 };
 
 /**
@@ -556,11 +720,17 @@ class DRUMSTICK_EXPORT ClientEvent : public SequencerEvent
 public:
     /** Default constructor */
     ClientEvent() : SequencerEvent() {}
-    /** Constructor from an ALSA event record */
-    ClientEvent(snd_seq_event_t* event) : SequencerEvent(event) {}
+    /**
+     * Constructor from an ALSA event record
+     * @param event an ALSA event record
+     */
+    explicit ClientEvent(const snd_seq_event_t* event) : SequencerEvent(event) {}
+    /**
+     * Gets the client number
+     * @return the client number
+     */
     int getClient() const { return m_event.data.addr.client; }
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(ClientEvent)
+    virtual ClientEvent* clone() const override;
 };
 
 /**
@@ -571,12 +741,17 @@ class DRUMSTICK_EXPORT PortEvent : public ClientEvent
 public:
     /** Default constructor */
     PortEvent() : ClientEvent() {}
-    /** Constructor from an ALSA event record */
-    PortEvent(snd_seq_event_t* event) : ClientEvent(event) {}
-    /** Gets the port number */
+    /**
+     * Constructor from an ALSA event record
+     * @param event an ALSA event record
+     */
+    explicit PortEvent(const snd_seq_event_t* event) : ClientEvent(event) {}
+    /**
+     * Gets the port number
+     * @return the port number
+     */
     int getPort() const { return m_event.data.addr.port; }
-    /** Clone this object returning a pointer to the new object */
-    CLONE_EVENT_DECLARATION(PortEvent)
+    virtual PortEvent* clone() const override;
 };
 
 /**
@@ -589,10 +764,9 @@ public:
     friend class MidiClient;
 
 public:
-    /** Default constructor */
     RemoveEvents();
     RemoveEvents(const RemoveEvents& other);
-    RemoveEvents(snd_seq_remove_events_t* other);
+    explicit RemoveEvents(snd_seq_remove_events_t* other);
     virtual ~RemoveEvents();
     RemoveEvents* clone();
     RemoveEvents& operator=(const RemoveEvents& other);
@@ -624,7 +798,7 @@ class DRUMSTICK_EXPORT MidiCodec : public QObject
 {
     Q_OBJECT
 public:
-    explicit MidiCodec(int bufsize, QObject* parent = 0);
+    explicit MidiCodec(int bufsize, QObject* parent = nullptr);
     ~MidiCodec();
 
     void init();
@@ -644,8 +818,8 @@ private:
     snd_midi_event_t* m_Info;
 };
 
-} /* namespace drumstick */
-
 /** @} */
 
+}} /* namespace drumstick::ALSA */
+
 #endif //DRUMSTICK_ALSAEVENT_H
